import axios from 'axios'
import { db } from "@/lib/db"
import { NextResponse } from "next/server";
import fetchUser from '@/lib/fetch-user';
import fetchUserWithImpersonation from '@/lib/fetch-user-impersonation';

export async function GET(req: Request) {
    try {
        // Verify state against user id in user table.
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        const connection = await db.twitchConnection.findFirst({
          where: {
            userId: user.id
          }
        })
        if (!connection) {
            return NextResponse.json({ message: 'You do not have permission for this.', error: null, value: null }, { status: 403 })
        }

        try {
            const { expires_in }: { client_id:string, login:string, scopes:string[], user_id:string, expires_in:number } = (await axios.get("https://id.twitch.tv/oauth2/validate", {
              headers: {
                Authorization: 'OAuth ' + connection.accessToken
              }
            })).data;

            if (expires_in > 3600) {
                let data = await db.twitchConnection.findFirst({
                    where: {
                      userId: user.id
                    }
                })

                let dataFormatted = {
                    user_id: user.id,
                    access_token: data?.accessToken,
                    refresh_token: data?.refreshToken,
                    broadcaster_id: connection.broadcasterId,
                    expires_in
                }
                return NextResponse.json(dataFormatted, { status: 201 });
            }
        } catch (error) {
        }

        // Post to https://id.twitch.tv/oauth2/token
        const token: { access_token:string, expires_in:number, refresh_token:string, token_type:string, scope:string[] } = (await axios.post("https://id.twitch.tv/oauth2/token", {
            client_id: process.env.TWITCH_BOT_CLIENT_ID,
            client_secret: process.env.TWITCH_BOT_CLIENT_SECRET,
            grant_type: "refresh_token",
            refresh_token: connection.refreshToken
        })).data

        // Fetch values from token.
        const { access_token, expires_in, refresh_token, token_type } = token

        if (!access_token || !refresh_token || token_type !== "bearer") {
            return NextResponse.json({ message: 'Unauthorized.', error: null, value: null }, { status: 401 });
        }

        await db.twitchConnection.update({
          where: {
            userId: user.id
          },
          data: {
            accessToken: access_token,
            refreshToken: refresh_token
          }
        })

        const data = {
            user_id: user.id,
            access_token,
            refresh_token,
            broadcaster_id: connection.broadcasterId,
            expires_in
        }
        
        return NextResponse.json(data)
    } catch (error) {
        console.log("[ACCOUNT]", error);
        return NextResponse.json({ message: 'Something went wrong', error: error, value: null }, { status: 500 })
    }
}